

/* Program cac18 */
/* Calculation of different phase equilibria */

/* Note that this program contains target calculations,
   which cannot be performed with the 'light' version of ChemApp */ 

#include "cacint.h"

int main()
{
  LI noerr, islite, iO2, iC, iN2;
  DB vals[2], TP[2], result, temp;

  /* Initialise ChemApp */ 
  tqini(&noerr);

  /* Open data-file for reading. In this case, a data-file containing
     the thermochemical data for the system carbon-nitrogen-oxygen is
     selected. */
  tqopna("cno.dat",10,&noerr);

  /* Read data-file */
  tqrfil(&noerr);
  
  /* Close data-file */
  tqclos(10,&noerr);


  /* Since we will perform an extensive property target calculation
     later, we will do the equilibrium calculations using
     streams. This way we can associate an initial state regarding
     temperature and pressure to the incoming substances, which will
     be taken into account when ChemApp calculates the extensive
     property (e.g. heat) balance. */

  /* Since we are going to prefer degrees Celsius as temperature unit,
     we will change the unit now */
  tqcsu("Temperature", "C", &noerr);

  /* Define a stream at room temperature and 1 bar */
  TP[0] = 20.0;
  TP[1] = 1.0;
  tqsttp("inputs", TP, &noerr);

  /* Set stream constituents: 0.5 mol of oxygen and 10 grams of carbon */

  /* Get the index number of the phase constituent oxygen ("O2") in
     the gas phase. If a thermochemical data-file for ChemApp and
     ChemSage contains a gas phase, it is always the first one: */
  tqinpc("O2", 1, &iO2, &noerr);

  /* Get the index number of the phase carbon ("C") */
  tqinp("C", &iC, &noerr);

  /* Set the incoming amount of oxygen in the stream (0.5 mol) */
  tqstca("inputs", 1, iO2, .5, &noerr);

  /* Change the amount unit to gram and set the incoming amount of
     carbon in the stream (10 gram) */
  tqcsu("Amount", "gram", &noerr);
  tqstca("inputs", iC, 0, 10.0, &noerr);

  /* Set the temperature for the equilibrium calculation (1500 C) */
  tqstec("T", 0, 1500.0, &noerr);

  /* Calculate the equilibrium */
  tqce(" ", 0, 0, vals, &noerr);

  /* Get the change in volume that takes place */
  tqgetr("V", 0, 0, &result, &noerr);
  printf("Change in volume/dm^3 is %g\n", result);


  /* Get the enthalpy change for the process (since its sign is
     negative, energy is released */
  tqgetr("H", 0, 0, &result, &noerr);
  printf("Change in enthalpy/J is %g\n", result);

    
  /* Get the activity of carbon, which, at 1500 C is far from being
     unity */
  tqgetr("ac", iC, 0, &result, &noerr);
  printf("Activity of carbon is %g\n", result);


  /* Once an equilibrium has been calculated using tqce/tqcel,
     subsequent calculations can be performed using tqcen/tqcenl,
     which speeds up the equilibrium calculation by taking results
     from the previous calculation as initial estimates */
  for (temp = 1510; temp <= 1600; temp +=10) {
    
    tqstec("T", 0, temp, &noerr);
    tqcen(" ", 0, 0, vals, &noerr);
    tqgetr("ac", iC, 0, &result, &noerr);
    printf("Activity of carbon at T = %g C is %g\n", temp, result);
  
  }



  /* Similar to tqcel, tqcenl also provides a ChemSage output table */  
  tqcenl(" ", 0, 0, vals, &noerr);


  /* Check if we are working with the 'light' version.
     If we do, omit the following target calculation(s). */
  tqlite(&islite, &noerr);
  if (islite) 
    {
      printf("*** Target calculations have been omitted here,\n"
             "*** since they are not possible with the\n"
             "*** 'light' version of ChemApp.\n\n");
    }
  else 
    {


  /* Performing target calculations */

  /* Note that target calculations are not possible with ChemApp
     "light", you need the regular version of ChemApp to run the
     examples below. */

  /* Performing target calculations requires that ChemApp needs to be
     told 2 things:
     - the target itself,
     - the target variable, i.e. which variable ChemApp is allowed to 
       vary to achieve the target.
     
     First we will perform a formation phase target calculation, which
     is used to determine the conditions under which a phase becomes
     stable, using carbon as an example. As a target variable we will
     use the temperature. The result will be the temperature under
     which solid carbon will just become stable (smoke point). */

  /* Define the target: phase C, amount of C = 0 */
  tqstec("a", iC, 0.0, &noerr);

  /* Define the target variable, which is done during the call to
     tqce.  The value passed as vals[0] will be taken as an initial
     estimate. */
  vals[0] = 500.0;
  tqce("T", 0, 0, vals, &noerr);

  /* Get the temperature, which ChemApp determined */
  tqgetr("T", 0, 0, &result, &noerr);
  printf("Calculated formation temperature of "
	 "carbon in degrees Celsius: %g\n", result);


  /* Check whether ChemApp calculated correctly by verifying that the
     activity of carbon is unity. */
  tqgetr("ac", iC, 0, &result, &noerr);
  printf("Activity of carbon is %g\n", result);


  /* The equilibrium amount of carbon should still be zero though, as
     we had ChemApp find the temperature where carbon _just_ gets
     stable */
  tqgetr("a", iC, 0, &result, &noerr);
  printf("Amount of carbon is %g\n", result);


  /* It is also possible to specify a different amount than zero for
     the phase formation target calculation. To to this, pass a
     different value to tqstec when defining the target. As an
     example, calculate the temperature where 1 gram of carbon is
     stable */
  tqstec("a", iC, 1.0, &noerr);
  tqce("T", 0, 0, vals, &noerr);
  tqgetr("T", 0, 0, &result, &noerr);
  printf("Calculated formation temperature for " 
         "1 gram of carbon in deg. Celsius: %g\n", result);


  }

  /* Before performing extensive property target calculations, tqremc
     is used to remove all conditions and targets set previously */
  tqremc(-2, &noerr);
    
  /* Remove also the stream "inputs" */
  tqstrm("inputs", &noerr);

  /* It is also useful to reset vals, which now still contains a
     starting value for a temperature in vals[0] */
  vals[0] = 0;

  /* If it is also desired to reset all units to default units, which
     would in this case reset the amount to mol and the temperature
     unit to Kelvin, call tqremc with a value of -1 */
  tqremc(-1, &noerr);

  /* Set up a new stream which contains 1 mol of pure oxygen,
     preheated to 400 K at 1 bar */
  TP[0] = 400.0;
  TP[1] = 1.0;
  tqsttp("oxygen", TP, &noerr);
  tqstca("oxygen", 1, iO2, 1.0, &noerr);
    
  /* Another stream that contains 1 mol of solid carbon at room
     temperature and 1 bar */
  TP[0] = 298.15;
  TP[1] = 1.0;
  tqsttp("carbon", TP, &noerr);
  tqstca("carbon", iC, 0, 1.0, &noerr);
    
  /* Calculate the equilibrium and retrieve the enthalpy released for
     this process */
  tqce(" ", 0, 0, vals, &noerr);
  tqgetr("H", 0, 0, &result, &noerr);
  printf("Change in enthalpy/J is %g\n", result);


  /* Check if we are working with the 'light' version.
     If we do, omit the following target calculation(s). */
  tqlite(&islite, &noerr);
  if (islite) 
    {
      printf("*** Target calculations have been omitted here,\n"
             "*** since they are not possible with the\n"
             "*** 'light' version of ChemApp.\n\n");
    }
  else 
    {


  /* Determine the adiabatic temperature for this process. This would
     be the temperature at which the above mentioned enthalpy is
     zero */

  /* Define the target: enthalpy, value = 0 */
  tqstec("H", 0, 0.0, &noerr);

  /* Define the target variable, which is done during the call to
     tqce. The value passed as vals[0] will be taken as an initial
     estimate. */
  vals[0] = 1000.0;
  tqce("T", 0, 0, vals, &noerr);

  /* Get the temperature ChemApp calculated (which is the adiabatic
     temperature in this example) */
  tqgetr("T", 0, 0, &result, &noerr);
  printf("Calculated adiabatic temperature/K for "
         "this process: %g\n", result);


  /* One can investigate how this temperature changes if one assumes a
     mixture that resembles air, instead of pure O2. Air has an N2/O2
     ratio of about 4:1. Keeping 1 mol of oxygen, we thus have to add
     4 mol of N2. */
  tqinpc("N2", 1, &iN2, &noerr);
  tqstca("oxygen", 1, iN2, 4.0, &noerr);
     
  tqce("T", 0, 0, vals, &noerr);

  tqgetr("T", 0, 0, &result, &noerr);
  printf("Adiabatic temperature/K for this process "
         "with a N2/O2 mixture: %g\n", result);


  }

  return 0;
 
}


