

C A simple process using streams

C Note that this program contains target calculations,
C which cannot be performed with the 'light' version of ChemApp

C The following function calculates the sum of all equilibrium amounts
C of the condensed phases. It starts with phase #2, since the gas phase
C is #1. The index number of the last phase to be considered is passed
C as an argument to the function, and is retrieved in the main program
C with a call to TQNOP.
      DOUBLE PRECISION FUNCTION GETACP(LASTPH)
      INTEGER LASTPH, I
      DOUBLE PRECISION AMOUNT
      GETACP= 0.D0

C Add up the equilibrium amounts of all condensed phases
      DO I=2, LASTPH
        CALL TQGETR('A ', I, 0, AMOUNT, NOERR)
        GETACP = GETACP + AMOUNT
      ENDDO
      END

C Main program
      PROGRAM CAF103
      IMPLICIT NONE
      
      INTEGER NOERR, ISLITE, NPHASE, J, K, ICO2, ICO, IO2, IN2
      DOUBLE PRECISION VALS(2), TP(2), RESULT, ACOND, AGAS, TEMP,
     *     IAO2, IAN2, GETACP

C Initialise ChemApp 
      CALL TQINI(NOERR)
      
C Open data-file for reading. In this case, a data-file containing the
C thermochemical data for the system carbon-nitrogen-oxygen is selected.
      CALL TQOPNA('cno.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C Since we will perform an extensive property target calculation later,
C we will use streams. This way we can assign an initial state
C regarding temperature and pressure to the incoming substances, which
C will be taken into account when ChemApp calculates the extensive
C property (e.g. heat) balance.

C Since we are going to use degrees Celsius as temperature unit
C throughout the calculations, we will change the unit now
      CALL TQCSU('Temperature ', 'C ', NOERR)

C According to the process scheme shown above, define the first stream.
C It is the incoming stream of mixed CO2 and CO at 200 C and 1 bar,
C which we will call 'CO2CO'. The amount of CO2 and CO in this stream is
C set to 0.5 and 2.5 mol, respectively.
      TP(1) = 200.D0
      TP(2) = 1.D0
      CALL TQSTTP('CO2CO ', TP, NOERR)
C Set the constituent amounts of CO2 and CO. To do this, get the index
C number of these constituents of the gas phase, which is phase number
C 1.
      CALL TQINPC('CO2 ', 1, ICO2, NOERR)
      CALL TQINPC('CO ', 1, ICO, NOERR)
C Use the index numbers just obtained to set the constituent amount of
C CO2 in the stream 'CO2CO' to 0.5 mol.
      CALL TQSTCA('CO2CO ', 1, ICO2, .5D0, NOERR)
C Similarly, set the amount of CO in the same stream to 2.5 mol.
      CALL TQSTCA('CO2CO ', 1, ICO, 2.5D0, NOERR)

C Define the second stream ('O2N2'), which is air supplied at room
C temperature and 1 bar for the combustion. The air is assumed to
C consist of O2 and N2 at a ratio of 1:4. The amount of O2 is initially
C set to 0.5 mol (thus N2 = 2.0 mol), and will later be varied.
      TP(1) = 25.D0
      TP(2) = 1.D0
      CALL TQSTTP('O2N2 ', TP, NOERR)
C Get the index numbers of O2 and N2.
      CALL TQINPC('O2 ', 1, IO2, NOERR)
      CALL TQINPC('N2 ', 1, IN2, NOERR)
C Set the amount of O2
      CALL TQSTCA('O2N2 ', 1, IO2, .5D0, NOERR)
C Set the amount of N2
      CALL TQSTCA('O2N2 ', 1, IN2, 2.0D0, NOERR)

C Set the temperature of the reactor at which the equilibrium reaction
C (i.e. the combustion) will take place. It is initially set to 600 C at
C first, but will be varied later.
      CALL TQSTEC('T ', 0, 600.D0, NOERR)

C In the result table to be printed out subsequently, the amount unit
C for the condensed phases formed should be 'gram', while the input was
C given in 'mol'.
      CALL TQCSU('Amount ', 'gram ', NOERR)

C Calculate the equilibrium and print out the result table for a quick
C overview on the equilibrium state.
      CALL TQCEL(' ', 0, 0, VALS, NOERR)


C As can be seen from the above table, both the gas phase and carbon are
C stable under the conditions chosen. It can also be seen that N2, CO2,
C and CO are the only majority gas phase species. All other gas phase
C species are produced in negligible amounts only. 

C The values for the amounts of condensed phases and the gas phase can
C be retrieved using TQGETR. Since the gas phase, if present in the
C data-file, has always the phase index number 1, this means that we
C have to add up all the amounts of the phases from phase #2 to the last
C one in order to retrieve the total amount of condensed phases.

C Find out how many phases we have in the data-file, and use this number
C in all subsequent calls to GETACP
      CALL TQNOP(NPHASE, NOERR)

C Get the total equilibrium amount of the condensed phases, using the
C function GETACP declared above.
      ACOND = GETACP(NPHASE)

 11   FORMAT(1X,A,G12.5,A)
      WRITE(UNIT=*,FMT=11) 'Amount of condensed material: ', 
     *     ACOND, ' gram'


C Retrieve the amount of gas phase. Note: The current amount unit is
C still 'gram'. If the amount of the gas phase is desired in 'mol'
C instead, TQCSU would need to be called again.
      CALL TQGETR('A ', 1, 0, AGAS, NOERR)

      WRITE(UNIT=*,FMT=11) 'Amount of gas phase: ', AGAS, ' gram'


C Retrieve values for some extensive properties.  Since we used streams
C for this example, these extensive property values are calculated
C relative to the incoming streams, when the whole system is selected.

C Get the enthalpy change associated with this reaction.
      CALL TQGETR('H ', 0, 0, RESULT, NOERR)
      WRITE(UNIT=*,FMT=11) 'Delta H: ', RESULT, ' J'

C The negative sign of this value indicates that energy is released by
C the system.

C Get the change in volume of the process, which is the volume of the
C equilibrium gas phase minus the sum of the volumes of the incoming
C streams.
      CALL TQGETR('V ', 0, 0, RESULT, NOERR)
      WRITE(UNIT=*,FMT=11) 'Delta V:', RESULT, ' dm^3'

C Note: using the option 'V' with TQGETR normally would retrieve the
C _total_ volume of the system. There is one exception, in which case it
C retrieves the _change_ in volume, and that is the case when streams
C are used _and_ the whole system (i.e. TQGETR is called using '0,0', as
C above) is selected. To retrieve the _total_ volume instead in the
C present case, use the option 'VT':
      CALL TQGETR('VT ', 0, 0, RESULT, NOERR)
      WRITE(UNIT=*,FMT=11) 'Total V:', RESULT, ' dm^3'


C Compare the two values for the volume with the corresponding ones in
C the output table produced earlier. The volume printed at the top of
C such an output table is always the _total_ volume, whereas the line
C specifying the extensive property balance toward the end of the table
C shows the _change_ in volume.

C TQSTXP can be used to retrieve extensive property values for the
C incoming streams. Get the volume of the incoming CO2/CO stream:
      CALL TQSTXP('CO2CO ', 'V ', RESULT, NOERR)
      WRITE(UNIT=*,FMT=11) 'V(CO2CO): ', RESULT, ' dm^3'
      
C Get the value of the incoming air stream:
      CALL TQSTXP('O2N2 ', 'V ', RESULT, NOERR)
      WRITE(UNIT=*,FMT=11) 'V(O2N2): ', RESULT, ' dm^3'


C For users of ChemApp versions prior to 3.2.0: Please note that the
C values returned by TQSTXP always refer to the _last_ calculated
C equilibrium state. Even if one uses TQSTXP to retrieve properties of
C _incoming_ streams, one thus needs to perform an equilibrium
C calculation before the correct values are obtained. If TQSTXP is
C called before the first equilibrium calculation has been performed
C using TQCE or TQCEL, VAL will be zero.

C As of version 3.2.0 of ChemApp, TQSTXP can also be called _before_ the
C equilibrium calculation)

C As can be seen from the above results, this process, under the
C conditions we have chosen, results in solids to be formed in the
C reactor. We will now further investigate this systems by looking at
C how the amount of condensed material changes when 
C a) the reaction temperature is varied, and 
C b) the amount of air supplied is varied.


C ****************************************************************
C Varying the reaction temperature
C ****************************************************************

C Since several individual equilibria will be calculated, the results
C (temperature and amount of condensed phases) will be stored to a file
C for further processing.
      OPEN(11, FILE='caf103_1.rst', STATUS='unknown', IOSTAT=NOERR)
      IF (NOERR .NE. 0) THEN
         WRITE(*,*) 'ERROR: Cannot open result file.'
         STOP
      ENDIF

C Vary the temperature between 200 C and 1000 C in steps of 5 C
      DO J=200, 1000, 5

C Use the variable TEMP to hold the current temperature.
         TEMP = J
         CALL TQSTEC('T ', 0, TEMP, NOERR)

C Calculate the equilibrium. In this case, we only need the value for the
C amount of condensed phases formed, therefore TQCE is used and not
C TQCEL.
         CALL TQCE(' ', 0, 0, VALS, NOERR)

C Get the amount of condensed phases
         ACOND = GETACP(NPHASE)

C Write the temperature and the amount of condensed phases to the result
C file
 12      FORMAT(2E12.5)
         WRITE(11,FMT=12) TEMP, ACOND
      ENDDO

C Close the result file 
      CLOSE(11)

C When plotted, the resulting graph looks like Figure ref(caf103_1)

C ****************************************************************
C Varying the amount of air supplied
C ****************************************************************

C Instead of the temperature, the amount of incoming air will be varied
C now. The temperature will be kept constant at the initial value of 
C 600 C. To plot the results later on, we will again store them in a
C separate file.
      OPEN(11, FILE='caf103_2.rst', STATUS='unknown', IOSTAT=NOERR)
      IF (NOERR .NE. 0) THEN
         WRITE(*,*) 'ERROR: Cannot open result file.'
         STOP
      ENDIF

C Set the temperature for the equilibrium calculation (600 C)
      CALL TQSTEC('T ', 0, 600.D0, NOERR)

C Calculate 201 equilibria...
      DO J=0, 200

C ... with the O2 amount varying between 0 and 1 mol...
         IAO2 = J * 0.005D0
C ... and the N2 amount between 0 and 4 mol.
         IAN2 = IAO2 * 4.D0

C To make sure ChemApp interprets these values as 'mol', set the amount
C unit accordingly
         CALL TQCSU('Amount ', 'mol ', NOERR)

C Set the amount of O2 in the air stream
         CALL TQSTCA('O2N2 ', 1, IO2, IAO2, NOERR)
C Set the amount of N2 in the air stream
         CALL TQSTCA('O2N2 ', 1, IN2, IAN2, NOERR)

C Calculate the equilibrium
         CALL TQCE(' ', 0, 0, VALS, NOERR)

C Set the amount unit to 'gram'
         CALL TQCSU('Amount ', 'gram ', NOERR)

C Get the amount of condensed phases
         ACOND = GETACP(NPHASE)

C Write the amounts of O2 and condensed phases to the result file
         WRITE(11,FMT=12) IAO2, ACOND
      ENDDO

C Close the result file 
      CLOSE(11)

C When plotted, the resulting graph looks like Figure ref(caf103_2)
C ****************************************************************
C Varying the amount of air supplied _and_ the temperature
C ****************************************************************

C To investigate the combined influence of temperature and amount of air
C supplied on the resulting amount of condensed phases, both parameters
C are varied in nested loops. The results are again stored in a file for
C further processing.
      OPEN(11, FILE='caf103_3.rst', STATUS='unknown', IOSTAT=NOERR)
      IF (NOERR .NE. 0) THEN
         WRITE(*,*) 'ERROR: Cannot open result file.'
         STOP
      ENDIF

C The outer loop will vary the temperature between 200 C and 1000 C
      DO J=200, 1000, 20

         TEMP = J
         CALL TQSTEC('T ', 0, TEMP, NOERR)

C The inner loop varies the amount of incoming air as above. 
         DO K=0, 200, 5

            IAO2 = K * 0.005D0
            IAN2 = IAO2 * 4.D0

            CALL TQCSU('Amount ', 'mol ', NOERR)
C Set the incoming amount of O2
            CALL TQSTCA('O2N2 ', 1, IO2, IAO2, NOERR)
C Set the incoming amount of N2
            CALL TQSTCA('O2N2 ', 1, IN2, IAN2, NOERR)

            CALL TQCE(' ', 0, 0, VALS, NOERR)

            CALL TQCSU('Amount ', 'gram ', NOERR)

C Get the amount of condensed phases
            ACOND = GETACP(NPHASE)

C Write the temperature and the amounts of O2 and condensed phases to
C the result file
 13         FORMAT(3E12.5)
            WRITE(11,FMT=13) TEMP, IAO2, ACOND

         ENDDO
         WRITE(11,*)

      ENDDO

C Close the result file 
      CLOSE(11)

C When plotted, the resulting graph looks like Figure ref(caf103_3)
C Check if we are working with the 'light' version.
C If we do, omit the following target calculation(s).
      CALL TQLITE(ISLITE, NOERR)
      IF (ISLITE .EQ. 1) THEN
         WRITE(*,FMT='(3(1X,A,/))') 
     *     '*** Target calculations have been omitted here,',
     *     '*** since they are not possible with the ',
     *     '*** ''light'' version of ChemApp.'

      ELSE

C ****************************************************************
C Varying the amount of air supplied _and_ the temperature, searching
C for the explicit condition where the amount of condensed phase is zero.
C ****************************************************************

C Assuming that one is interested in the above process _not_ producing
C any condensed phases (i.e. complete combustion), it would be helpful
C to have a diagram which corresponds to the bottom plane of the above
C figure: A graph which, as a function of the incoming amount of air,
C shows the temperature at which the amount of condensed phases formed
C is zero.

C For the calculation procedure with ChemApp, this results in the
C following steps:
C 1) We are looking for the temperature at which condensed phases _just_
C    start to form. In terms of ChemApp, we thus have to define a
C    precipitation target with respect to the gas phase.
C 2) For each value of the amount of air supplied, define the 'O2N2'
C    stream with the proper amounts of O2 and N2.
C 3) ChemApp is then supposed to vary the temperature until the target
C    (a phase precipitates from the gas phase) is met. Temperature is
C    thus the target variable. This is specified as a parameter to TQCE,
C    together with an estimated value.
C 4) The equilibrium is calculated for each of the amounts of air
C    supplied, then the equilibrium temperature is retrieved and stored in
C    the result file.
      OPEN(11, FILE='caf103_4.rst', STATUS='unknown', IOSTAT=NOERR)
      IF (NOERR .NE. 0) THEN
         WRITE(*,*) 'ERROR: Cannot open result file.'
         STOP
      ENDIF

C Reset the amount unit to 'mol'
      CALL TQCSU('Amount ', 'mol ', NOERR)

C Define the target as a precipitation target ('-1') for the gas
C phase (phase index 1)
      CALL TQSTEC('A ', 1, -1.D0, NOERR) 
      
C The initial estimate for the temperature is passed via the first
C element of VALS to subsequent calls to TQCE
      VALS(1) = 600.D0

C Vary the amount of supplied air
      DO J=0, 200

         IAO2 = J * 0.005D0
         IAN2 = IAO2 * 4.D0
C Set the incoming amount of O2
         CALL TQSTCA('O2N2 ', 1, IO2, IAO2, NOERR)
C Set the incoming amount of N2
         CALL TQSTCA('O2N2 ', 1, IN2, IAN2, NOERR)

C Define the target variable (temperature), with the initial estimate in
C the first element of VALS, and calculate the equilibrium
         CALL TQCE('T ', 0, 0, VALS, NOERR)
         
C Retrieve the calculated equilibrium temperature
         CALL TQGETR('T ', 0, 0, TEMP, NOERR)

C Store the amount of O2 supplied and the calculated equilibrium
C temperature to the result file
         WRITE(11,FMT=12) IAO2, TEMP
      ENDDO

C Close data-file 
      CLOSE(11)

C When plotted, the resulting graph looks like Figure ref(caf103_4)
      ENDIF

      END


