
C Performing volume target calculations instead of calculations at
C constant volume

C Note that this program contains target calculations,
C which cannot be performed with the 'light' version of ChemApp

      PROGRAM CAF210
      IMPLICIT NONE
      
      INTEGER NOERR, NUMCON, ISIO2, IC, ISLITE
      DOUBLE PRECISION VALS(2), RESULT

C Initialise ChemApp 
      CALL TQINI(NOERR)
      
C Open data-file for reading 
      CALL TQOPNA('cosi.dat', 10, NOERR)

C Read data-file 
      CALL TQRFIL(NOERR)

C Close data-file 
      CALL TQCLOS(10, NOERR)

C 1) Equilibrium calculation at constant volume. While ChemApp will be
C able to converge and find a solution for this particular calculation,
C similar calculations with more complex data-files, especially those
C containing molar volume data, might make it impossible for ChemApp to
C find the equilibrium. For this reason, the use of constant volume
C calculations using the option 'VT' with TQSETC is discouraged,
C instead, use the calculation method outlined below in 2). Future
C versions of ChemApp will have the option 'VT' disabled.

C Set conditions for an equilibrium calculation at constant volume:
C temperature, incoming amounts, and volume.

C Set temperature to 1900 K
      CALL TQSETC('T ', 0, 0, 1900.D0, NUMCON, NOERR)

C Set incoming amounts: 1 mol SiO2(quartz) and 2 mol C
C Get phase index numbers for both phases and set their amounts
      CALL TQINP('SiO2(quartz)', ISIO2, NOERR)
      CALL TQSETC('IA ', ISIO2, 1, 1.D0, NUMCON, NOERR)

      CALL TQINP('C ', IC, NOERR)
      CALL TQSETC('IA ', IC, 0, 2.D0, NUMCON, NOERR)

C Use option 'VT' to set the volume to 100 dm^3. Setting the volume to a
C constant value means ChemApp will calculate the resulting pressure.
      CALL TQSETC('VT ', 0, 0, 100.D0, NUMCON, NOERR)

C Calculate the equilibrium.
      CALL TQCEL(' ', 0, 0, VALS, NOERR)


C Use TQGETR with its option 'VT' to get the calculated total
C volume. 
      CALL TQGETR('VT ', 0, 0, RESULT, NOERR)

 10   FORMAT(1X,A,G12.5,A)
      WRITE(*,FMT=10) 'Equilibrium volume (total): ', RESULT, ' dm^3'


C Check if we are working with the 'light' version.
C If we do, omit the following target calculation(s).
      CALL TQLITE(ISLITE, NOERR)
      IF (ISLITE .EQ. 1) THEN
         WRITE(*,FMT='(3(1X,A,/))') 
     *     '*** Target calculations have been omitted here,',
     *     '*** since they are not possible with the ',
     *     '*** ''light'' version of ChemApp.'

      ELSE

C 2) Equilibrium calculation with volume as target and pressure as
C target variable. This is the recommended method, as ChemApp is able to
C take discontinuities in volume properly into account and obtain a
C converged solution. The only difference in defining this calculation
C compared to the one above is the use of option 'V' instead of 'VT',
C and the specification of the target variable (pressure) when calling
C TQCEL.

C First, reset all previous conditions
      CALL TQREMC(-2, NOERR)
      
C Set temperature to 1900 K
      CALL TQSETC('T ', 0, 0, 1900.D0, NUMCON, NOERR)

C Set incoming amounts: 1 mol SiO2(quartz) and 2 mol C
      CALL TQSETC('IA ', ISIO2, 1, 1.D0, NUMCON, NOERR)
      CALL TQSETC('IA ', IC, 0, 2.D0, NUMCON, NOERR)

C Use option 'V' to define a volume of 100 dm^3 as the target for the
C next equilibrium calculation
      CALL TQSETC('V ', 0, 0, 100.D0, NUMCON, NOERR)

C Calculate the equilibrium, using the pressure as target variable with
C an initial estimate of 1 bar
      VALS(1) = 1.D0
      CALL TQCEL('P ', 0, 0, VALS, NOERR)


C As you can see from the two output tables produced by the calls to
C TQCEL, the only difference is the asterisk that marks the target
C variable (pressure) in the second table. All calculated values are
C otherwise the same.

C Use TQGETR with its option 'VT' to get the calculated total
C volume. Note that TQGETR's option 'VT' is _not_ related to
C TQSETC/TQSTEC's option 'VT'. The latter only specifies that the total
C volume of the system should be _retrieved_, and can always be
C used. The use of TQGETR's option 'VT' need not be changed in any way
C or discontinued with any future version of ChemApp.
      CALL TQGETR('VT ', 0, 0, RESULT, NOERR)

      WRITE(*,FMT=10) 'Equilibrium volume (total): ', RESULT, ' dm^3'


      ENDIF
      END

